# from dh_potluck.email.email_service import EmailService
#
# # EmailService.send_email_html(
# #     [
# #         {
# #             'email': 'jake.delmas@dashhudson.com',
# #             'name': 'jake delmas',
# #             'type': 'to'
# #         },
# #         {
# #             'email': 'jakemdelmas@gmail.com',
# #             'name': 'jake delmas',
# #             'type': 'cc'
# #         }
# #     ],
# #     subject='Test Subject',
# #     html='<b>this is a test</b>',
# # )
# # EmailService.send_email_template(
# #     [
# #         {
# #             'email': 'jake.delmas@dashhudson.com',
# #             'name': 'jake delmas',
# #             'type': 'to'
# #         },
# #         {
# #             'email': 'jakemdelmas@gmail.com',
# #             'name': 'jake delmas',
# #             'type': 'cc'
# #         }
# #     ],
# #     'restore-password',
# #     [{'name': 'link', 'content': 'test'}, {'name': 'name', 'content': 'JAKEEE'}],
# #     subject='template test',
# # )
#
# test = 'dddd' 'ccccc'
# print(test)
